@@ -325,12 +325,16 @@ public interface Context extends Container, ContextBind {
 
     /**
      * Return the alternate Deployment Descriptor name.
+     *
+     * @return the name
      */
     public String getAltDDName();
 
 
     /**
      * Set an alternate Deployment Descriptor name.
+     *
+     * @param altDDName The new name
      */
     public void setAltDDName(String altDDName) ;
 
@@ -345,6 +349,8 @@ public interface Context extends Container, ContextBind {
 
     /**
      * Return the deny-uncovered-http-methods flag for this web application.
+     *
+     * @return The current value of the flag
      */
     public boolean getDenyUncoveredHttpMethods();
 
@@ -359,6 +365,8 @@ public interface Context extends Container, ContextBind {
 
     /**
      * Return the display name of this web application.
+     *
+     * @return The display name
      */
     public String getDisplayName();
 
@@ -372,7 +380,9 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the distributable flag for this web application.
+     * Get the distributable flag for this web application.
+     *
+     * @return The value of the distributable flag for this web application.
      */
     public boolean getDistributable();
 
@@ -386,8 +396,9 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the document root for this Context.  This can be an absolute
-     * pathname, a relative pathname, or a URL.
+     * Obtain the document root for this Context.
+     *
+     * @return An absolute pathname, a relative pathname, or a URL.
      */
     public String getDocBase();
 
@@ -402,13 +413,18 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the URL encoded context path, using UTF-8.
+     * Return the URL encoded context path
+     *
+     * @return The URL encoded (with UTF-8) context path
      */
     public String getEncodedPath();
 
 
     /**
-     * Return the boolean on the annotations parsing.
+     * Determine if annotations parsing is currently disabled
+     *
+     * @return {@code true} if annotation parsing is disabled for this web
+     *         application
      */
     public boolean getIgnoreAnnotations();
 
@@ -423,7 +439,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the login configuration descriptor for this web application.
+     * @return the login configuration descriptor for this web application.
      */
     public LoginConfig getLoginConfig();
 
@@ -437,7 +453,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the naming resources associated with this web application.
+     * @return the naming resources associated with this web application.
      */
     public NamingResourcesImpl getNamingResources();
 
@@ -451,7 +467,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the context path for this web application.
+     * @return the context path for this web application.
      */
     public String getPath();
 
@@ -465,7 +481,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the public identifier of the deployment descriptor DTD that is
+     * @return the public identifier of the deployment descriptor DTD that is
      * currently being parsed.
      */
     public String getPublicId();
@@ -481,7 +497,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the reloadable flag for this web application.
+     * @return the reloadable flag for this web application.
      */
     public boolean getReloadable();
 
@@ -495,7 +511,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the override flag for this web application.
+     * @return the override flag for this web application.
      */
     public boolean getOverride();
 
@@ -509,7 +525,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the privileged flag for this web application.
+     * @return the privileged flag for this web application.
      */
     public boolean getPrivileged();
 
@@ -523,13 +539,13 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the servlet context for which this Context is a facade.
+     * @return the Servlet context for which this Context is a facade.
      */
     public ServletContext getServletContext();
 
 
     /**
-     * Return the default session timeout (in minutes) for this
+     * @return the default session timeout (in minutes) for this
      * web application.
      */
     public int getSessionTimeout();
@@ -564,7 +580,7 @@ public interface Context extends Container, ContextBind {
     public void setSwallowAbortedUploads(boolean swallowAbortedUploads);
 
     /**
-     * Return the value of the swallowOutput flag.
+     * @return the value of the swallowOutput flag.
      */
     public boolean getSwallowOutput();
 
@@ -580,7 +596,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the Java class name of the Wrapper implementation used
+     * @return the Java class name of the Wrapper implementation used
      * for servlets registered in this Context.
      */
     public String getWrapperClass();
@@ -683,7 +699,7 @@ public interface Context extends Container, ContextBind {
     public void setJarScanner(JarScanner jarScanner);
 
     /**
-     * Obtain the {@link Authenticator} that is used by this context or
+     * @return the {@link Authenticator} that is used by this context or
      * <code>null</code> if none is used.
      */
     public Authenticator getAuthenticator();
@@ -691,21 +707,29 @@ public interface Context extends Container, ContextBind {
     /**
      * Set whether or not the effective web.xml for this context should be
      * logged on context start.
+     *
+     * @param logEffectiveWebXml set to <code>true</code> to log the complete
+     *        web.xml that will be used for the webapp
      */
     public void setLogEffectiveWebXml(boolean logEffectiveWebXml);
 
     /**
      * Should the effective web.xml for this context be logged on context start?
+     *
+     * @return true if the reconstructed web.xml that will be used for the
+     *   webapp should be logged
      */
     public boolean getLogEffectiveWebXml();
 
     /**
-     * Get the instance manager associated with this context.
+     * @return the instance manager associated with this context.
      */
     public InstanceManager getInstanceManager();
 
     /**
      * Set the instance manager associated with this context.
+     *
+     * @param instanceManager the new instance manager instance
      */
     public void setInstanceManager(InstanceManager instanceManager);
 
@@ -756,6 +780,8 @@ public interface Context extends Container, ContextBind {
 
     /**
      * Add a security constraint to the set for this web application.
+     *
+     * @param constraint The security constraint that should be added
      */
     public void addConstraint(SecurityConstraint constraint);
 
@@ -795,17 +821,6 @@ public interface Context extends Container, ContextBind {
      */
     public void addFilterMapBefore(FilterMap filterMap);
 
-    /**
-     * Add the classname of an InstanceListener to be added to each
-     * Wrapper appended to this Context.
-     *
-     * @param listener Java class name of an InstanceListener class
-     *
-     * @deprecated Will be removed in 8.5.x onwards
-     */
-    @Deprecated
-    public void addInstanceListener(String listener);
-
 
     /**
      * Add a Locale Encoding Mapping (see Sec 5.4 of Servlet spec 2.4)
@@ -948,32 +963,34 @@ public interface Context extends Container, ContextBind {
      * the Java implementation class appropriate for this Context
      * implementation.  The constructor of the instantiated Wrapper
      * will have been called, but no properties will have been set.
+     *
+     * @return a newly created wrapper instance that is used to wrap a Servlet
      */
     public Wrapper createWrapper();
 
 
     /**
-     * Return the set of application listener class names configured
+     * @return the set of application listener class names configured
      * for this application.
      */
     public String[] findApplicationListeners();
 
 
     /**
-     * Return the set of application parameters for this application.
+     * @return the set of application parameters for this application.
      */
     public ApplicationParameter[] findApplicationParameters();
 
 
     /**
-     * Return the set of security constraints for this web application.
+     * @return the set of security constraints for this web application.
      * If there are none, a zero-length array is returned.
      */
     public SecurityConstraint[] findConstraints();
 
 
     /**
-     * Return the error page entry for the specified HTTP error code,
+     * @return the error page entry for the specified HTTP error code,
      * if any; otherwise return <code>null</code>.
      *
      * @param errorCode Error code to look up
@@ -982,7 +999,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the error page entry for the specified Java exception type,
+     * @return the error page entry for the specified Java exception type,
      * if any; otherwise return <code>null</code>.
      *
      * @param exceptionType Exception type to look up
@@ -992,14 +1009,14 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the set of defined error pages for all specified error codes
+     * @return the set of defined error pages for all specified error codes
      * and exception types.
      */
     public ErrorPage[] findErrorPages();
 
 
     /**
-     * Return the filter definition for the specified filter name, if any;
+     * @return the filter definition for the specified filter name, if any;
      * otherwise return <code>null</code>.
      *
      * @param filterName Filter name to look up
@@ -1008,29 +1025,19 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the set of defined filters for this Context.
+     * @return the set of defined filters for this Context.
      */
     public FilterDef[] findFilterDefs();
 
 
     /**
-     * Return the set of filter mappings for this Context.
+     * @return the set of filter mappings for this Context.
      */
     public FilterMap[] findFilterMaps();
 
 
     /**
-     * Return the set of InstanceListener classes that will be added to
-     * newly created Wrappers automatically.
-     *
-     * @deprecated Will be removed in 8.5.x onwards
-     */
-    @Deprecated
-    public String[] findInstanceListeners();
-
-
-    /**
-     * Return the MIME type to which the specified extension is mapped,
+     * @return the MIME type to which the specified extension is mapped,
      * if any; otherwise return <code>null</code>.
      *
      * @param extension Extension to map to a MIME type
@@ -1039,14 +1046,14 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the extensions for which MIME mappings are defined.  If there
+     * @return the extensions for which MIME mappings are defined.  If there
      * are none, a zero-length array is returned.
      */
     public String[] findMimeMappings();
 
 
     /**
-     * Return the value for the specified context initialization
+     * @return the value for the specified context initialization
      * parameter name, if any; otherwise return <code>null</code>.
      *
      * @param name Name of the parameter to return
@@ -1055,7 +1062,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the names of all defined context initialization parameters
+     * @return the names of all defined context initialization parameters
      * for this Context.  If no parameters are defined, a zero-length
      * array is returned.
      */
@@ -1068,12 +1075,13 @@ public interface Context extends Container, ContextBind {
      * is one.  Otherwise, return the specified role unchanged.
      *
      * @param role Security role to map
+     * @return The role name that was mapped to the specified role
      */
     public String findRoleMapping(String role);
 
 
     /**
-     * Return <code>true</code> if the specified security role is defined
+     * @return <code>true</code> if the specified security role is defined
      * for this application; otherwise return <code>false</code>.
      *
      * @param role Security role to verify
@@ -1082,14 +1090,14 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the security roles defined for this application.  If none
+     * @return the security roles defined for this application.  If none
      * have been defined, a zero-length array is returned.
      */
     public String[] findSecurityRoles();
 
 
     /**
-     * Return the servlet name mapped by the specified pattern (if any);
+     * @return the servlet name mapped by the specified pattern (if any);
      * otherwise return <code>null</code>.
      *
      * @param pattern Pattern for which a mapping is requested
@@ -1098,14 +1106,14 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the patterns of all defined servlet mappings for this
+     * @return the patterns of all defined servlet mappings for this
      * Context.  If no mappings are defined, a zero-length array is returned.
      */
     public String[] findServletMappings();
 
 
     /**
-     * Return the context-relative URI of the error page for the specified
+     * @return the context-relative URI of the error page for the specified
      * HTTP status code, if any; otherwise return <code>null</code>.
      *
      * @param status HTTP status code to look up
@@ -1114,7 +1122,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the set of HTTP status codes for which error pages have
+     * @return the set of HTTP status codes for which error pages have
      * been specified.  If none are specified, a zero-length array
      * is returned.
      */
@@ -1122,26 +1130,29 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Get the associated ThreadBindingListener.
+     * @return the associated ThreadBindingListener.
      */
     public ThreadBindingListener getThreadBindingListener();
 
 
     /**
      * Get the associated ThreadBindingListener.
+     *
+     * @param threadBindingListener Set the listener that will receive
+     *       notifications when entering and exiting the application scope
      */
     public void setThreadBindingListener(ThreadBindingListener threadBindingListener);
 
 
     /**
-     * Return the set of watched resources for this Context. If none are
+     * @return the set of watched resources for this Context. If none are
      * defined, a zero length array will be returned.
      */
     public String[] findWatchedResources();
 
 
     /**
-     * Return <code>true</code> if the specified welcome file is defined
+     * @return <code>true</code> if the specified welcome file is defined
      * for this Context; otherwise return <code>false</code>.
      *
      * @param name Welcome file to verify
@@ -1150,21 +1161,21 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the set of welcome files defined for this Context.  If none are
+     * @return the set of welcome files defined for this Context.  If none are
      * defined, a zero-length array is returned.
      */
     public String[] findWelcomeFiles();
 
 
     /**
-     * Return the set of LifecycleListener classes that will be added to
+     * @return the set of LifecycleListener classes that will be added to
      * newly created Wrappers automatically.
      */
     public String[] findWrapperLifecycles();
 
 
     /**
-     * Return the set of ContainerListener classes that will be added to
+     * @return the set of ContainerListener classes that will be added to
      * newly created Wrappers automatically.
      */
     public String[] findWrapperListeners();
@@ -1173,6 +1184,8 @@ public interface Context extends Container, ContextBind {
     /**
      * Notify all {@link javax.servlet.ServletRequestListener}s that a request
      * has started.
+     *
+     * @param request The request object that will be passed to the listener
      * @return <code>true</code> if the listeners fire successfully, else
      *         <code>false</code>
      */
@@ -1181,6 +1194,8 @@ public interface Context extends Container, ContextBind {
     /**
      * Notify all {@link javax.servlet.ServletRequestListener}s that a request
      * has ended.
+     *
+     * @param request The request object that will be passed to the listener
      * @return <code>true</code> if the listeners fire successfully, else
      *         <code>false</code>
      */
@@ -1247,18 +1262,6 @@ public interface Context extends Container, ContextBind {
     public void removeFilterMap(FilterMap filterMap);
 
 
-    /**
-     * Remove a class name from the set of InstanceListener classes that
-     * will be added to newly created Wrappers.
-     *
-     * @param listener Class name of an InstanceListener class to be removed
-     *
-     * @deprecated Will be removed in 8.5.x onwards
-     */
-    @Deprecated
-    public void removeInstanceListener(String listener);
-
-
     /**
      * Remove the MIME mapping for the specified extension, if it exists;
      * otherwise, no action is taken.
@@ -1339,7 +1342,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the real path for a given virtual path, if possible; otherwise
+     * @return the real path for a given virtual path, if possible; otherwise
      * return <code>null</code>.
      *
      * @param path The path to the desired resource
@@ -1348,7 +1351,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * Return the effective major version of the Servlet spec used by this
+     * @return the effective major version of the Servlet spec used by this
      * context.
      */
     public int getEffectiveMajorVersion();
@@ -1357,12 +1360,14 @@ public interface Context extends Container, ContextBind {
     /**
      * Set the effective major version of the Servlet spec used by this
      * context.
+     *
+     * @param major Set the version number
      */
     public void setEffectiveMajorVersion(int major);
 
 
     /**
-     * Return the effective minor version of the Servlet spec used by this
+     * @return the effective minor version of the Servlet spec used by this
      * context.
      */
     public int getEffectiveMinorVersion();
@@ -1371,12 +1376,14 @@ public interface Context extends Container, ContextBind {
     /**
      * Set the effective minor version of the Servlet spec used by this
      * context.
+     *
+     * @param minor Set the version number
      */
     public void setEffectiveMinorVersion(int minor);
 
 
     /**
-     * Obtain the JSP configuration for this context.
+     * @return the JSP configuration for this context.
      * Will be null if there is no JSP configuration.
      */
     public JspConfigDescriptor getJspConfigDescriptor();
@@ -1384,6 +1391,8 @@ public interface Context extends Container, ContextBind {
     /**
      * Set the JspConfigDescriptor for this context.
      * A null value indicates there is not JSP configuration.
+     *
+     * @param descriptor the new JSP configuration
      */
     public void setJspConfigDescriptor(JspConfigDescriptor descriptor);
 
@@ -1399,20 +1408,24 @@ public interface Context extends Container, ContextBind {
 
     /**
      * Is this Context paused whilst it is reloaded?
+     *
+     * @return <code>true</code> if the context has been paused
      */
     public boolean getPaused();
 
 
     /**
      * Is this context using version 2.2 of the Servlet spec?
+     *
+     * @return <code>true</code> for a legacy Servlet 2.2 webapp
      */
     boolean isServlet22();
 
     /**
-     * Notification that servlet security has been dynamically set in a
+     * Notification that Servlet security has been dynamically set in a
      * {@link javax.servlet.ServletRegistration.Dynamic}
-     * @param registration servlet security was modified for
-     * @param servletSecurityElement new security constraints for this servlet
+     * @param registration Servlet security was modified for
+     * @param servletSecurityElement new security constraints for this Servlet
      * @return urls currently mapped to this registration that are already
      *         present in web.xml
      */
@@ -1423,6 +1436,8 @@ public interface Context extends Container, ContextBind {
      * Sets the (comma separated) list of Servlets that expect a resource to be
      * present. Used to ensure that welcome files associated with Servlets that
      * expect a resource to be present are not mapped when there is no resource.
+     *
+     * @param resourceOnlyServlets The Servlet names comma separated list
      */
     public void setResourceOnlyServlets(String resourceOnlyServlets);
 
@@ -1443,7 +1458,7 @@ public interface Context extends Container, ContextBind {
     public boolean isResourceOnlyServlet(String servletName);
 
     /**
-     * Return the base name to use for WARs, directories or context.xml files
+     * @return the base name to use for WARs, directories or context.xml files
      * for this context.
      */
     public String getBaseName();
@@ -1452,24 +1467,29 @@ public interface Context extends Container, ContextBind {
      * Set the version of this web application - used to differentiate
      * different versions of the same web application when using parallel
      * deployment.
+     *
+     * @param webappVersion The webapp version associated with the context,
+     *    which should be unique
      */
     public void setWebappVersion(String webappVersion);
 
     /**
-     * Set the version of this web application - used to differentiate
+     * @return The version of this web application, used to differentiate
      * different versions of the same web application when using parallel
-     * deployment. If not specified, defaults to the empty string.
+     * deployment.
      */
     public String getWebappVersion();
 
     /**
      * Configure whether or not requests listeners will be fired on forwards for
      * this Context.
+     *
+     * @param enable <code>true</code> to fire request listeners when forwarding
      */
     public void setFireRequestListenersOnForwards(boolean enable);
 
     /**
-     * Determine whether or not requests listeners will be fired on forwards for
+     * @return whether or not requests listeners will be fired on forwards for
      * this Context.
      */
     public boolean getFireRequestListenersOnForwards();
@@ -1478,11 +1498,14 @@ public interface Context extends Container, ContextBind {
      * Configures if a user presents authentication credentials, whether the
      * context will process them when the request is for a non-protected
      * resource.
+     *
+     * @param enable <code>true</code> to perform authentication even outside
+     *    security constraints
      */
     public void setPreemptiveAuthentication(boolean enable);
 
     /**
-     * Determines if a user presents authentication credentials, will the
+     * @return if a user presents authentication credentials, will the
      * context will process them when the request is for a non-protected
      * resource.
      */
@@ -1491,17 +1514,19 @@ public interface Context extends Container, ContextBind {
     /**
      * Configures if a response body is included when a redirect response is
      * sent to the client.
+     *
+     * @param enable <code>true</code> to send a response body for redirects
      */
     public void setSendRedirectBody(boolean enable);
 
     /**
-     * Determines if the context is configured to include a response body as
+     * @return if the context is configured to include a response body as
      * part of a redirect response.
      */
     public boolean getSendRedirectBody();
 
     /**
-     * Return the Loader with which this Context is associated.
+     * @return the Loader with which this Context is associated.
      */
     public Loader getLoader();
 
@@ -1513,7 +1538,7 @@ public interface Context extends Container, ContextBind {
     public void setLoader(Loader loader);
 
     /**
-     * Return the Resources with which this Context is associated.
+     * @return the Resources with which this Context is associated.
      */
     public WebResourceRoot getResources();
 
@@ -1525,7 +1550,7 @@ public interface Context extends Container, ContextBind {
     public void setResources(WebResourceRoot resources);
 
     /**
-     * Return the Manager with which this Context is associated.  If there is
+     * @return the Manager with which this Context is associated.  If there is
      * no associated Manager, return <code>null</code>.
      */
     public Manager getManager();
@@ -1548,7 +1573,7 @@ public interface Context extends Container, ContextBind {
     public void setAddWebinfClassesResources(boolean addWebinfClassesResources);
 
     /**
-     * Gets the flag that indicates if /WEB-INF/classes should be treated like
+     * @return the flag that indicates if /WEB-INF/classes should be treated like
      * an exploded JAR and JAR resources made available as if they were in a
      * JAR.
      */
@@ -1652,7 +1677,7 @@ public interface Context extends Container, ContextBind {
     public Map<String, String> findPreDestroyMethods();
 
     /**
-     * Obtain the token necessary for operations on the associated JNDI naming
+     * @return  the token necessary for operations on the associated JNDI naming
      * context.
      */
     public Object getNamingToken();
@@ -1669,7 +1694,7 @@ public interface Context extends Container, ContextBind {
     public void setCookieProcessor(CookieProcessor cookieProcessor);
 
     /**
-     * Obtains the {@link CookieProcessor} that will be used to process cookies
+     * @return the {@link CookieProcessor} that will be used to process cookies
      * for this Context.
      */
     public CookieProcessor getCookieProcessor();
