@@ -788,7 +788,7 @@ public class ApplicationContextFacade implements ServletContext {
     /**
      * Use reflection to invoke the requested method. Cache the method object
      * to speed up the process
-     * @param appContext The AppliationContext object on which the method
+     * @param appContext The ApplicationContext object on which the method
      *                   will be invoked
      * @param methodName The method to call.
      * @param params The arguments passed to the called method.
@@ -846,7 +846,7 @@ public class ApplicationContextFacade implements ServletContext {
     /**
      * Executes the method of the specified <code>ApplicationContext</code>
      * @param method The method object to be invoked.
-     * @param context The AppliationContext object on which the method
+     * @param context The ApplicationContext object on which the method
      *                   will be invoked
      * @param params The arguments passed to the called method.
      */
@@ -858,12 +858,8 @@ public class ApplicationContextFacade implements ServletContext {
                    InvocationTargetException {
 
         if (SecurityUtil.isPackageProtectionEnabled()){
-           return AccessController.doPrivileged(new PrivilegedExceptionAction<Object>(){
-                @Override
-                public Object run() throws IllegalAccessException, InvocationTargetException{
-                    return method.invoke(context,  params);
-                }
-            });
+           return AccessController.doPrivileged(
+                   new PrivilegedExecuteMethod(method, context,  params));
         } else {
             return method.invoke(context, params);
         }
@@ -895,4 +891,23 @@ public class ApplicationContextFacade implements ServletContext {
 
         throw realException;
     }
+
+
+    private static class PrivilegedExecuteMethod implements PrivilegedExceptionAction<Object> {
+
+        private final Method method;
+        private final ApplicationContext context;
+        private final Object[] params;
+
+        public PrivilegedExecuteMethod(Method method, ApplicationContext context, Object[] params) {
+            this.method = method;
+            this.context = context;
+            this.params = params;
+        }
+
+        @Override
+        public Object run() throws Exception {
+            return method.invoke(context, params);
+        }
+    }
 }
