@@ -27,7 +27,6 @@ import javax.servlet.http.HttpServletResponse;
 import org.apache.catalina.Context;
 import org.apache.catalina.Globals;
 import org.apache.catalina.Wrapper;
-import org.apache.catalina.comet.CometEvent;
 import org.apache.catalina.connector.ClientAbortException;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
@@ -148,13 +147,10 @@ final class StandardHostValve extends ValveBase {
                 }
             } catch (Throwable t) {
                 ExceptionUtils.handleThrowable(t);
+                container.getLogger().error("Exception Processing " + request.getRequestURI(), t);
                 // If a new error occurred while trying to report a previous
-                // error simply log the new error and allow the original error
-                // to be reported.
-                if (response.isErrorReportRequired()) {
-                    container.getLogger().error("Exception Processing " +
-                            request.getRequestURI(), t);
-                } else {
+                // error allow the original error to be reported.
+                if (!response.isErrorReportRequired()) {
                     request.setAttribute(RequestDispatcher.ERROR_EXCEPTION, t);
                     throwable(request, response, t);
                 }
@@ -197,52 +193,6 @@ final class StandardHostValve extends ValveBase {
     }
 
 
-    /**
-     * Process Comet event.
-     *
-     * @param request Request to be processed
-     * @param response Response to be produced
-     * @param event the event
-     *
-     * @exception IOException if an input/output error occurred
-     * @exception ServletException if a servlet error occurred
-     */
-    @Override
-    public final void event(Request request, Response response, CometEvent event)
-        throws IOException, ServletException {
-
-        // Select the Context to be used for this Request
-        Context context = request.getContext();
-
-        context.bind(false, MY_CLASSLOADER);
-
-        // Ask this Context to process this request
-        context.getPipeline().getFirst().event(request, response, event);
-
-
-        // Error page processing
-        response.setSuspended(false);
-
-        Throwable t = (Throwable) request.getAttribute(
-                RequestDispatcher.ERROR_EXCEPTION);
-
-        if (t != null) {
-            throwable(request, response, t);
-        } else {
-            status(request, response);
-        }
-
-        // Access a session (if present) to update last accessed time, based on a
-        // strict interpretation of the specification
-        if (ACCESS_SESSION) {
-            request.getSession(false);
-        }
-
-        // Restore the context classloader
-        context.unbind(false, MY_CLASSLOADER);
-    }
-
-
     // -------------------------------------------------------- Private Methods
 
     /**
@@ -449,13 +399,13 @@ final class StandardHostValve extends ValveBase {
             }
 
             // Indicate that we have successfully processed this custom page
-            return (true);
+            return true;
 
         } catch (Throwable t) {
             ExceptionUtils.handleThrowable(t);
             // Report our failure to process this custom page
             container.getLogger().error("Exception Processing " + errorPage, t);
-            return (false);
+            return false;
 
         }
     }
