@@ -45,7 +45,7 @@ public class ReplicatedContext extends StandardContext implements MapOwner {
     private int mapSendOptions = Channel.SEND_OPTIONS_DEFAULT;
     private static final Log log = LogFactory.getLog( ReplicatedContext.class );
     protected static final long DEFAULT_REPL_TIMEOUT = 15000;//15 seconds
-    private static final StringManager sm = StringManager.getManager(ReplicatedContext.class.getPackage().getName());
+    private static final StringManager sm = StringManager.getManager(ReplicatedContext.class);
 
     /**
      * Start this component and implement the requirements
@@ -90,7 +90,7 @@ public class ReplicatedContext extends StandardContext implements MapOwner {
 
         super.stopInternal();
 
-        if ( map!=null && map instanceof ReplicatedMap) {
+        if (map instanceof ReplicatedMap) {
             ((ReplicatedMap<?, ?>) map).breakdown();
         }
 
