@@ -118,13 +118,13 @@ public class Constants {
             "<table cellspacing=\"4\" border=\"0\">\n" +
             " <tr>\n" +
             "  <td colspan=\"2\">\n" +
-            "   <a href=\"http://www.apache.org/\">\n" +
-            "    <img border=\"0\" alt=\"The Apache Software Foundation\" align=\"left\"\n" +
-            "         src=\"{0}/images/asf-logo.gif\">\n" +
-            "   </a>\n" +
             "   <a href=\"http://tomcat.apache.org/\">\n" +
             "    <img border=\"0\" alt=\"The Tomcat Servlet/JSP Container\"\n" +
-            "         align=\"right\" src=\"{0}/images/tomcat.gif\">\n" +
+            "         align=\"left\" src=\"{0}/images/tomcat.gif\">\n" +
+            "   </a>\n" +
+            "   <a href=\"http://www.apache.org/\">\n" +
+            "    <img border=\"0\" alt=\"The Apache Software Foundation\" align=\"right\"\n" +
+            "         src=\"{0}/images/asf-logo.svg\" style=\"width: 266px; height: 83px;\">\n" +
             "   </a>\n" +
             "  </td>\n" +
             " </tr>\n" +
@@ -201,7 +201,7 @@ public class Constants {
         HTML_TAIL_SECTION =
             "<hr size=\"1\" noshade=\"noshade\">\n" +
             "<center><font size=\"-1\" color=\"#525D76\">\n" +
-            " <em>Copyright &copy; 1999-2016, Apache Software Foundation</em>" +
+            " <em>Copyright &copy; 1999-2017, Apache Software Foundation</em>" +
             "</font></center>\n" +
             "\n" +
             "</body>\n" +
