@@ -62,8 +62,7 @@ import org.apache.tomcat.util.res.StringManager;
 public class StandardRoot extends LifecycleMBeanBase implements WebResourceRoot {
 
     private static final Log log = LogFactory.getLog(StandardRoot.class);
-    protected static final StringManager sm =
-            StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(StandardRoot.class);
 
     private Context context;
     private boolean allowLinking = false;
@@ -238,7 +237,7 @@ public class StandardRoot extends LifecycleMBeanBase implements WebResourceRoot
      * can be normalized without stepping outside of the root.
      *
      * @param path
-     * @return  the normlized path
+     * @return  the normalized path
      */
     private String validate(String path) {
         if (!getState().isAvailable()) {
@@ -361,6 +360,8 @@ public class StandardRoot extends LifecycleMBeanBase implements WebResourceRoot
         return result;
     }
 
+    // TODO: Should the createWebResourceSet() methods be removed to some
+    //       utility class for file system based resource sets?
 
     @Override
     public void createWebResourceSet(ResourceSetType type, String webAppMount,
@@ -562,10 +563,10 @@ public class StandardRoot extends LifecycleMBeanBase implements WebResourceRoot
         this.context = context;
     }
 
-    /*
+    /**
      * Class loader resources are handled by treating JARs in WEB-INF/lib as
      * resource JARs (without the internal META-INF/resources/ prefix) mounted
-     * at WEB-INF/claasses (rather than the web app root). This enables reuse
+     * at WEB-INF/classes (rather than the web app root). This enables reuse
      * of the resource handling plumbing.
      *
      * These resources are marked as class loader only so they are only used in
@@ -585,7 +586,8 @@ public class StandardRoot extends LifecycleMBeanBase implements WebResourceRoot
     }
 
     /**
-     * For unit testing
+     * For unit testing.
+     * @param main The main resources
      */
     protected final void setMainResources(WebResourceSet main) {
         this.main = main;
