@@ -199,19 +199,11 @@ public final class EmbeddedServletOptions implements Options {
      */
     private int jspIdleTimeout = -1;
 
-    /**
-     * System property that controls if the strict quoting rules are applied
-     * when parsing attribute values that use scriptlet expressions (<%=...%>).
-     */
-    private static final boolean STRICT_QUOTE_ESCAPING_DEFAULT= Boolean.parseBoolean(
-            System.getProperty(
-                    "org.apache.jasper.compiler.Parser.STRICT_QUOTE_ESCAPING",
-                    "true"));
     /**
      * Should JSP.1.6 be applied strictly to attributes defined using scriptlet
      * expressions?
      */
-    private boolean strictQuoteEscaping = STRICT_QUOTE_ESCAPING_DEFAULT;
+    private boolean strictQuoteEscaping = true;
 
     /**
      * When EL is used in JSP attribute values, should the rules for quoting of
@@ -479,6 +471,8 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Create an EmbeddedServletOptions object using data available from
      * ServletConfig and ServletContext.
+     * @param config The Servlet config
+     * @param context The Servlet context
      */
     public EmbeddedServletOptions(ServletConfig config,
             ServletContext context) {
@@ -787,8 +781,7 @@ public final class EmbeddedServletOptions implements Options {
                 this.strictQuoteEscaping = false;
             } else {
                 if (log.isWarnEnabled()) {
-                    log.warn(Localizer.getMessage("jsp.warning.strictQuoteEscaping",
-                            Boolean.toString(STRICT_QUOTE_ESCAPING_DEFAULT)));
+                    log.warn(Localizer.getMessage("jsp.warning.strictQuoteEscaping"));
                 }
             }
         }
