@@ -18,13 +18,13 @@ package org.apache.jasper.compiler;
 
 import java.io.FileNotFoundException;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.util.Stack;
 
 import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
-import org.apache.jasper.xmlparser.XMLEncodingDetector;
-import org.apache.tomcat.util.scan.Jar;
+import org.apache.tomcat.Jar;
 import org.xml.sax.Attributes;
 
 /**
@@ -88,9 +88,14 @@ class ParserController implements TagConstants {
      * Parses a JSP page or tag file. This is invoked by the compiler.
      *
      * @param inFileName The path to the JSP page or tag file to be parsed.
+     *
+     * @return The parsed nodes
+     *
+     * @throws JasperException If an error occurs during parsing
+     * @throws IOException If an I/O error occurs such as the file not being
+     *         found
      */
-    public Node.Nodes parse(String inFileName)
-    throws FileNotFoundException, JasperException, IOException {
+    public Node.Nodes parse(String inFileName) throws JasperException, IOException {
         // If we're parsing a packaged tag file or a resource included by it
         // (using an include directive), ctxt.getTagFileJar() returns the
         // JAR file from which to read the tag file or included resource,
@@ -105,9 +110,14 @@ class ParserController implements TagConstants {
      * compiler.
      *
      * @param inFileName The path to the JSP page or tag file to be parsed.
+     *
+     * @return The parsed directive nodes
+     *
+     * @throws JasperException If an error occurs during parsing
+     * @throws IOException If an I/O error occurs such as the file not being
+     *         found
      */
-    public Node.Nodes parseDirectives(String inFileName)
-    throws FileNotFoundException, JasperException, IOException {
+    public Node.Nodes parseDirectives(String inFileName) throws JasperException, IOException {
         // If we're parsing a packaged tag file or a resource included by it
         // (using an include directive), ctxt.getTagFileJar() returns the
         // JAR file from which to read the tag file or included resource,
@@ -125,9 +135,15 @@ class ParserController implements TagConstants {
      * @param parent The parent node of the include directive.
      * @param jar The JAR file from which to read the included resource,
      * or null of the included resource is to be read from the filesystem
+     *
+     * @return The parsed nodes
+     *
+     * @throws JasperException If an error occurs during parsing
+     * @throws IOException If an I/O error occurs such as the file not being
+     *         found
      */
     public Node.Nodes parse(String inFileName, Node parent, Jar jar)
-    throws FileNotFoundException, JasperException, IOException {
+            throws JasperException, IOException {
         // For files that are statically included, isTagfile and directiveOnly
         // remain unchanged.
         return doParse(inFileName, parent, jar);
@@ -140,9 +156,15 @@ class ParserController implements TagConstants {
      *
      * @param inFileName    The name of the tag file to be parsed.
      * @param jar The location of the tag file.
+     *
+     * @return The parsed tag file nodes
+     *
+     * @throws JasperException If an error occurs during parsing
+     * @throws IOException If an I/O error occurs such as the file not being
+     *         found
      */
     public Node.Nodes parseTagFileDirectives(String inFileName, Jar jar)
-            throws FileNotFoundException, JasperException, IOException {
+            throws JasperException, IOException {
         boolean isTagFileSave = isTagFile;
         boolean directiveOnlySave = directiveOnly;
         isTagFile = true;
@@ -159,11 +181,6 @@ class ParserController implements TagConstants {
      * @param inFileName The name of the JSP page or tag file to be parsed.
      * @param parent The parent node (non-null when processing an include
      * directive)
-     * @param isTagFile true if file to be parsed is tag file, and false if it
-     * is a regular JSP page
-     * @param directivesOnly true if the file to be parsed is a tag file and
-     * we are only interested in the directives needed for constructing a
-     * TagFileInfo.
      * @param jar  The JAR file from which to read the JSP page or tag file,
      * or null if the JSP page or tag file is to be read from the filesystem
      */
@@ -301,16 +318,13 @@ class ParserController implements TagConstants {
             sourceEnc = "ISO-8859-1";
         } else {
             // XML syntax or unknown, (auto)detect encoding ...
-            Object[] ret = XMLEncodingDetector.getEncoding(absFileName, jar,
-                    ctxt, err);
-            sourceEnc = (String) ret[0];
-            if (((Boolean) ret[1]).booleanValue()) {
-                isEncodingSpecifiedInProlog = true;
-            }
-            if (((Boolean) ret[2]).booleanValue()) {
-                isBomPresent = true;
-            }
-            skip = ((Integer) ret[3]).intValue();
+            InputStream inStream = JspUtil.getInputStream(absFileName, jar, ctxt);
+            EncodingDetector encodingDetector = new EncodingDetector(inStream);
+
+            sourceEnc = encodingDetector.getBomEncoding();
+            isEncodingSpecifiedInProlog = (encodingDetector.getPrologEncoding() != null);
+            isBomPresent = (encodingDetector.getSkip() > 0);
+            skip = encodingDetector.getSkip();
 
             if (!isXml && sourceEnc.equals("UTF-8")) {
                 /*
