@@ -41,7 +41,6 @@ import javax.naming.spi.ObjectFactory;
  * of your server.xml configuration file.
  * <p>
  * Example:
- * <p>
  * <pre>
  * &lt;Resource name="mail/send" auth="CONTAINER"
  *           type="javax.mail.internet.MimePartDataSource"/&gt;
@@ -76,7 +75,7 @@ import javax.naming.spi.ObjectFactory;
 public class SendMailFactory implements ObjectFactory
 {
     // The class name for the javamail MimeMessageDataSource
-    protected final String DataSourceClassName =
+    protected static final String DataSourceClassName =
         "javax.mail.internet.MimePartDataSource";
 
     @Override
