@@ -26,9 +26,12 @@ import static org.junit.Assert.fail;
 
 import org.junit.Test;
 
+import org.apache.catalina.Context;
+import org.apache.catalina.Host;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.Session;
 import org.apache.catalina.core.StandardContext;
+import org.apache.catalina.core.StandardHost;
 
 /**
  * Named Benchmarks so it is not automatically executed as part of the unit
@@ -152,8 +155,8 @@ public class Benchmarks {
      * 16 threads - ~45,600ms
      */
     @Test
-    public void testManagerBaseCreateSession() {
-        doTestManagerBaseCreateSession(1, 1000000);
+    public void testManagerBaseCreateSession() throws LifecycleException {
+        doTestManagerBaseCreateSession(1, 100000);
         doTestManagerBaseCreateSession(2, 1000000);
         doTestManagerBaseCreateSession(4, 1000000);
         doTestManagerBaseCreateSession(16, 1000000);
@@ -164,16 +167,18 @@ public class Benchmarks {
 
 
     private void doTestManagerBaseCreateSession(int threadCount,
-            int iterCount) {
+            int iterCount) throws LifecycleException {
 
         // Create a default session manager
         StandardManager mgr = new StandardManager();
-        try {
-            mgr.startInternal();
-        } catch (LifecycleException e) {
-            // Ignore - this is expected
-        }
-        mgr.setContext(new StandardContext());
+        mgr.setPathname(null);
+        Host host = new StandardHost();
+        host.setName("unittest");
+        Context context = new StandardContext();
+        context.setPath("");
+        context.setParent(host);
+        mgr.setContext(context);
+        mgr.start();
         mgr.generateSessionId();
         while (mgr.sessionCreationTiming.size() <
                 ManagerBase.TIMING_STATS_CACHE_SIZE) {
