@@ -115,6 +115,11 @@ public class TesterContext implements Context {
         return log;
     }
 
+    @Override
+    public String getLogName() {
+        return null;
+    }
+
     @Override
     public ObjectName getObjectName() {
         return null;
@@ -734,11 +739,6 @@ public class TesterContext implements Context {
         // NO-OP
     }
 
-    @Override
-    public void addInstanceListener(String listener) {
-        // NO-OP
-    }
-
     @Override
     public void addLocaleEncodingMappingParameter(String locale, String encoding) {
         // NO-OP
@@ -760,13 +760,11 @@ public class TesterContext implements Context {
     }
 
     @Override
-    @Deprecated
     public void addServletMapping(String pattern, String name) {
         // NO-OP
     }
 
     @Override
-    @Deprecated
     public void addServletMapping(String pattern, String name,
             boolean jspWildcard) {
         // NO-OP
@@ -848,11 +846,6 @@ public class TesterContext implements Context {
         return null;
     }
 
-    @Override
-    public String[] findInstanceListeners() {
-        return null;
-    }
-
     @Override
     public String findMimeMapping(String extension) {
         return null;
@@ -963,11 +956,6 @@ public class TesterContext implements Context {
         // NO-OP
     }
 
-    @Override
-    public void removeInstanceListener(String listener) {
-        // NO-OP
-    }
-
     @Override
     public void removeMimeMapping(String extension) {
         // NO-OP
